#ifndef ENTRY_H
#define ENTRY_H
/**
 * Generic class that wraps all the different types of information
 * that can be stored in a Slot.
 * @author Brian Demsky <bdemsky@uci.edu>
 * @version 1.0
 */

#include "Liveness.h"
#include "common.h"

#define TypeCommitPart 1
#define TypeAbort 2
#define TypeTransactionPart 3
#define TypeNewKey 4
#define TypeLastMessage 5
#define TypeRejectedMessage 6
#define TypeTableStatus 7
#define TypeSlot 8

class Entry : public Liveness {
	/* Records whether the information is still live or has been
	   superceded by a newer update.  */
private:
	bool islive;
protected:
	Slot *parentslot;

public:
	Entry(Slot *_parentslot) : islive(true), parentslot(_parentslot) {}

	/**
	 * Returns true if the Entry object is still live.
	 */
	bool isLive() { return islive; }

	/**
	 * Flags the entry object as dead.  Also decrements the live count
	 * of the parent slot.
	 */
	void setDead();

	/**
	 * Serializes the Entry object into the char buffer.
	 */
	virtual void encode(ByteBuffer *bb) = 0;

	/**
	 * Returns the size in chars the entry object will take in the char
	 * array.
	 */
	virtual int getSize() = 0;


	/**
	 * Returns a copy of the Entry that can be added to a different slot.
	 */
	virtual Entry *getCopy(Slot *s) = 0;
	friend Entry *Entry_decode(Slot *slot, ByteBuffer *bb);
};

/**
 * Static method for decoding char array into Entry objects.  First
 * char tells the type of entry.
 */
Entry *Entry_decode(Slot *slot, ByteBuffer *bb);

#endif
