#ifndef PENDINGTRANSACTION_H
#define PENDINGTRANSACTION_H

#include "common.h"

class PendingTransaction {
private:
	Hashset<KeyValue *> *keyValueUpdateSet;
	Hashset<KeyValue *> *keyValueGuardSet;
	int64_t arbitrator;
	int64_t clientLocalSequenceNumber;
	int64_t machineId;
	int32_t currentDataSize;

public:
	PendingTransaction(int64_t _machineId);
	~PendingTransaction();
	/**
	 * Add a new key value to the updates
	 *
	 */
	void addKV(KeyValue *newKV);
	/**
	 * Add a new key value to the guard set
	 *
	 */
	void addKVGuard(KeyValue *newKV);
	/**
	 * Checks if the arbitrator is the same
	 */
	bool checkArbitrator(int64_t arb);
	/**
	 * Get the transaction arbitrator
	 */
	int64_t getArbitrator();
	/**
	 * Get the key value update set
	 */
	Hashset<KeyValue *> *getKVUpdates() { return keyValueUpdateSet; }

	/**
	 * Get the key value update set
	 */
	Hashset<KeyValue *> *getKVGuard() { return keyValueGuardSet; }

	void setClientLocalSequenceNumber(int64_t _clientLocalSequenceNumber) { clientLocalSequenceNumber = _clientLocalSequenceNumber; }

	int64_t getClientLocalSequenceNumber() { return clientLocalSequenceNumber; }

	int64_t getMachineId() { return machineId; }

	bool evaluateGuard(Hashtable<IoTString *, KeyValue *> *keyValTableCommitted, Hashtable<IoTString *, KeyValue *> *keyValTableSpeculative, Hashtable<IoTString *, KeyValue *> *keyValTablePendingTransSpeculative);

	Transaction *createTransaction();

	Array<char> *convertDataToBytes();
};
#endif
